@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2006 Imendio AB
- * Copyright (C) 2003-2004 Geert-Jan Van den Bogaerde <geertjan@gnome.org>
- * Copyright (C) 2004      Martyn Russell <mr@gnome.org>
+ * Copyright (C) 2002-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,19 +16,25 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Richard Hult <richard@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
+ *          Geert-Jan Van den Bogaerde <geertjan@gnome.org>
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
 #include <stdlib.h>
 
-#include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <glade/glade.h>
 
 #include <libgossip/gossip-debug.h>
-#include <libgossip/gossip-conf.h>
 #include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-conf.h>
 
 #include "gossip-app.h"
 #include "gossip-chat.h"
@@ -38,6 +42,7 @@
 #include "gossip-geometry.h"
 #include "gossip-marshal.h"
 #include "gossip-preferences.h"
+#include "gossip-smiley.h"
 #include "gossip-spell.h"
 #include "gossip-spell-dialog.h"
 #include "gossip-ui-utils.h"
@@ -56,6 +61,9 @@ struct _GossipChatPriv {
 
 	GtkTooltips      *tooltips;
 
+	GSList           *sent_messages;
+	gint              sent_messages_index;
+
 	/* Used to automatically shrink a window that has temporarily
 	 * grown due to long input. 
 	 */
@@ -133,9 +141,9 @@ gossip_chat_class_init (GossipChatClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GossipChatClass, new_message),
 			      NULL, NULL,
-			      gossip_marshal_VOID__OBJECT,
+			      gossip_marshal_VOID__OBJECT_BOOLEAN,
 			      G_TYPE_NONE,
-			      1, GOSSIP_TYPE_MESSAGE);
+			      2, GOSSIP_TYPE_MESSAGE, G_TYPE_BOOLEAN);
 
 	chat_signals[NAME_CHANGED] =
 		g_signal_new ("name-changed",
@@ -181,9 +189,12 @@ gossip_chat_init (GossipChat *chat)
 		      NULL);
 
 	priv = GET_PRIV (chat);
-
+	
 	priv->tooltips = gtk_tooltips_new ();
 
+	priv->sent_messages = NULL;
+	priv->sent_messages_index = -1;
+
 	priv->default_window_height = -1;
 	priv->vscroll_visible = FALSE;
 
@@ -224,6 +235,9 @@ chat_finalize (GObject *object)
 	chat = GOSSIP_CHAT (object);
 	priv = GET_PRIV (chat);
 
+	g_slist_foreach (priv->sent_messages, (GFunc) g_free, NULL);
+	g_slist_free (priv->sent_messages);
+	
 	G_OBJECT_CLASS (gossip_chat_parent_class)->finalize (object);
 }
 
@@ -383,7 +397,7 @@ chat_text_view_size_allocate_cb (GtkWidget     *widget,
 		GtkWidget *vscroll;
 
 		priv->vscroll_visible = TRUE;
-		gtk_widget_set_size_request (sw, -1, MAX_INPUT_HEIGHT);
+		gtk_widget_set_size_request (sw, sw->allocation.width, MAX_INPUT_HEIGHT);
 		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
 						GTK_POLICY_NEVER,
 						GTK_POLICY_AUTOMATIC);
@@ -970,3 +984,99 @@ finished:
 
 	return ret_val;
 }
+
+void 
+gossip_chat_sent_message_add (GossipChat  *chat,
+			      const gchar *str)
+{
+	GossipChatPriv *priv;
+	GSList         *list;
+	GSList         *item;
+
+	g_return_if_fail (GOSSIP_IS_CHAT (chat));
+	g_return_if_fail (!G_STR_EMPTY (str));
+
+	priv = GET_PRIV (chat);
+
+	/* Save the sent message in our repeat buffer */
+	list = priv->sent_messages;
+	
+	/* Remove any other occurances of this msg */
+	while ((item = g_slist_find_custom (list, str, (GCompareFunc) strcmp)) != NULL) {
+		list = g_slist_remove_link (list, item);
+		g_free (item->data);
+		g_slist_free1 (item);
+	}
+
+	/* Trim the list to the last 10 items */
+	while (g_slist_length (list) > 10) {
+		item = g_slist_last (list);
+		if (item) {
+			list = g_slist_remove_link (list, item);
+			g_free (item->data);
+			g_slist_free1 (item);
+		}
+	}
+
+	/* Add new message */
+	list = g_slist_prepend (list, g_strdup (str));
+
+	/* Set list and reset the index */
+	priv->sent_messages = list;
+	priv->sent_messages_index = -1;
+}
+
+const gchar *
+gossip_chat_sent_message_get_next (GossipChat *chat)
+{
+	GossipChatPriv *priv;
+	gint            max;
+
+	g_return_val_if_fail (GOSSIP_IS_CHAT (chat), NULL);
+	
+	priv = GET_PRIV (chat);
+
+	if (!priv->sent_messages) {
+		gossip_debug (DEBUG_DOMAIN, 
+			      "No sent messages, next message is NULL");
+		return NULL;
+	}
+
+	max = g_slist_length (priv->sent_messages) - 1;
+
+	if (priv->sent_messages_index < max) {
+		priv->sent_messages_index++;
+	}
+	
+	gossip_debug (DEBUG_DOMAIN, 
+		      "Returning next message index:%d",
+		      priv->sent_messages_index);
+
+	return g_slist_nth_data (priv->sent_messages, priv->sent_messages_index);
+}
+
+const gchar *
+gossip_chat_sent_message_get_last (GossipChat *chat)
+{
+	GossipChatPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_CHAT (chat), NULL);
+
+	priv = GET_PRIV (chat);
+	
+	if (!priv->sent_messages) {
+		gossip_debug (DEBUG_DOMAIN, 
+			      "No sent messages, last message is NULL");
+		return NULL;
+	}
+
+	if (priv->sent_messages_index >= 0) {
+		priv->sent_messages_index--;
+	}
+
+	gossip_debug (DEBUG_DOMAIN, 
+		      "Returning last message index:%d",
+		      priv->sent_messages_index);
+
+	return g_slist_nth_data (priv->sent_messages, priv->sent_messages_index);
+}
