@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2006 Imendio AB
+ * Copyright (C) 2006-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,71 +18,86 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
 #include <stdlib.h>
+
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <glade/glade.h>
 
+#include <libgossip/gossip-account.h>
+#include <libgossip/gossip-log.h>
+#include <libgossip/gossip-message.h>
+#include <libgossip/gossip-session.h>
 #include <libgossip/gossip-debug.h>
+#include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-time.h>
 
 #include "gossip-app.h"
 #include "gossip-account-chooser.h"
-#include "gossip-log.h"
+#include "gossip-glade.h"
 #include "gossip-log-window.h"
 #include "gossip-ui-utils.h"
 
 #define DEBUG_DOMAIN "LogWindow"
 
 typedef struct {
-	GtkWidget      *window;
+	GtkWidget        *window;
 
-	GtkWidget      *notebook;
+	GtkWidget        *notebook;
 
-	GtkWidget      *entry_search;
-	GtkWidget      *button_search;
-	GtkWidget      *treeview_search;
-	GtkWidget      *scrolledwindow_search;
-	GossipChatView *chatview_search;
+	GtkWidget        *entry_find;
+	GtkWidget        *button_find;
+	GtkWidget        *treeview_find;
+	GtkWidget        *scrolledwindow_find;
+	GossipChatView   *chatview_find;
+	GtkWidget        *button_previous;
+	GtkWidget        *button_next;
 
-	GtkWidget      *vbox_contacts;
-	GtkWidget      *account_chooser_contacts;
-	GtkWidget      *entry_contacts;
-	GtkWidget      *calendar_contacts;
-	GtkWidget      *treeview_contacts;
-	GtkWidget      *scrolledwindow_contacts;
-	GossipChatView *chatview_contacts;
+	GtkWidget        *vbox_contacts;
+	GtkWidget        *account_chooser_contacts;
+	GtkWidget        *entry_contacts;
+	GtkWidget        *calendar_contacts;
+	GtkWidget        *treeview_contacts;
+	GtkWidget        *scrolledwindow_contacts;
+	GossipChatView   *chatview_contacts;
 
-	GtkWidget      *vbox_chatrooms;
-	GtkWidget      *account_chooser_chatrooms;
-	GtkWidget      *entry_chatrooms;
-	GtkWidget      *calendar_chatrooms;
-	GtkWidget      *treeview_chatrooms;
-	GtkWidget      *scrolledwindow_chatrooms;
-	GossipChatView *chatview_chatrooms;
+	GtkWidget        *vbox_chatrooms;
+	GtkWidget        *account_chooser_chatrooms;
+	GtkWidget        *entry_chatrooms;
+	GtkWidget        *calendar_chatrooms;
+	GtkWidget        *treeview_chatrooms;
+	GtkWidget        *scrolledwindow_chatrooms;
+	GossipChatView   *chatview_chatrooms;
 
-	gchar          *last_search;
+	gchar            *last_find;
+
+	GossipLogManager *log_manager;
 } GossipLogWindow;
 
 /* Searching */
-static void            log_window_search_text_data_func               (GtkCellLayout    *cell_layout,
+static void            log_window_find_text_data_func                 (GtkCellLayout    *cell_layout,
 								       GtkCellRenderer  *cell,
 								       GtkTreeModel     *tree_model,
 								       GtkTreeIter      *iter,
 								       GossipLogWindow  *window);
-static void            log_window_search_pixbuf_data_func             (GtkCellLayout    *cell_layout,
+static void            log_window_find_pixbuf_data_func               (GtkCellLayout    *cell_layout,
 								       GtkCellRenderer  *cell,
 								       GtkTreeModel     *tree_model,
 								       GtkTreeIter      *iter,
 								       GossipLogWindow  *window);
-static void            log_window_entry_search_activate_cb            (GtkWidget        *entry,
+static void            log_window_find_changed_cb                     (GtkTreeSelection *selection,
 								       GossipLogWindow  *window);
-static void            log_window_search_changed_cb                   (GtkTreeSelection *selection,
-								       GossipLogWindow  *window);
-static void            log_window_search_populate                     (GossipLogWindow  *window,
+static void            log_window_find_populate                       (GossipLogWindow  *window,
 								       const gchar      *search_criteria);
-static void            log_window_search_setup                        (GossipLogWindow  *window);
-static void            log_window_button_search_clicked_cb            (GtkWidget        *widget,
+static void            log_window_find_setup                          (GossipLogWindow  *window);
+static void            log_window_button_find_clicked_cb              (GtkWidget        *widget,
+								       GossipLogWindow  *window);
+static void            log_window_button_next_clicked_cb              (GtkWidget        *widget,
+								       GossipLogWindow  *window);
+static void            log_window_button_previous_clicked_cb          (GtkWidget        *widget,
 								       GossipLogWindow  *window);
 
 /* Contacts */
@@ -158,13 +173,13 @@ static void            log_window_destroy_cb                          (GtkWidget
 								       GossipLogWindow  *window);
 
 enum {
-	COL_SEARCH_STATUS,
-	COL_SEARCH_ACCOUNT,
-	COL_SEARCH_CONTACT,
-	COL_SEARCH_CONTACT_NAME,
-	COL_SEARCH_DATE,
-	COL_SEARCH_DATE_READABLE,
-	COL_SEARCH_COUNT
+	COL_FIND_STATUS,
+	COL_FIND_ACCOUNT,
+	COL_FIND_CONTACT,
+	COL_FIND_CONTACT_NAME,
+	COL_FIND_DATE,
+	COL_FIND_DATE_READABLE,
+	COL_FIND_COUNT
 };
 
 enum {
@@ -185,22 +200,24 @@ enum {
  * Search code.
  */
 static void
-log_window_search_pixbuf_data_func (GtkCellLayout   *cell_layout,
-				    GtkCellRenderer *cell,
-				    GtkTreeModel    *tree_model,
-				    GtkTreeIter     *iter,
-				    GossipLogWindow *window)
+log_window_find_pixbuf_data_func (GtkCellLayout   *cell_layout,
+				  GtkCellRenderer *cell,
+				  GtkTreeModel    *tree_model,
+				  GtkTreeIter     *iter,
+				  GossipLogWindow *window)
 {
 	GossipAccount *account;
 	GossipSession *session;
 	GdkPixbuf     *pixbuf;
 	gboolean       online;
 
-	gtk_tree_model_get (tree_model, iter, COL_SEARCH_ACCOUNT, &account, -1);
+	gtk_tree_model_get (tree_model, iter, COL_FIND_ACCOUNT, &account, -1);
 
 	session = gossip_app_get_session ();
 	online = gossip_session_is_connected (session, account);
-	pixbuf = gossip_pixbuf_from_account_status (account, GTK_ICON_SIZE_MENU, online);
+	pixbuf = gossip_account_status_create_pixbuf (account, 
+						      GTK_ICON_SIZE_MENU,
+						      online);
 
 	g_object_set (cell, "pixbuf", pixbuf, NULL);
 	g_object_set (cell, "visible", TRUE, NULL);
@@ -210,82 +227,90 @@ log_window_search_pixbuf_data_func (GtkCellLayout   *cell_layout,
 }
 
 static void
-log_window_search_text_data_func (GtkCellLayout   *cell_layout,
-				  GtkCellRenderer *cell,
-				  GtkTreeModel    *tree_model,
-				  GtkTreeIter     *iter,
-				  GossipLogWindow *window)
+log_window_find_text_data_func (GtkCellLayout   *cell_layout,
+				GtkCellRenderer *cell,
+				GtkTreeModel    *tree_model,
+				GtkTreeIter     *iter,
+				GossipLogWindow *window)
 {
 	GossipAccount *account;
 
-	gtk_tree_model_get (tree_model, iter, COL_SEARCH_ACCOUNT, &account, -1);
+	gtk_tree_model_get (tree_model, iter, COL_FIND_ACCOUNT, &account, -1);
 
 	g_object_set (cell, "text", gossip_account_get_name (account), NULL);
 	g_object_unref (account);
 }
 
 static void
-log_window_entry_search_activate_cb (GtkWidget       *entry,
-				     GossipLogWindow *window)
+log_window_entry_find_changed_cb (GtkWidget       *entry,
+				  GossipLogWindow *window)
 {
 	const gchar *str;
+	gboolean     is_sensitive = TRUE;
 
-	str = gtk_entry_get_text (GTK_ENTRY (window->entry_search));
+	str = gtk_entry_get_text (GTK_ENTRY (window->entry_find));
 
-	g_free (window->last_search);
-	window->last_search = g_strdup (str);
+	is_sensitive &= !G_STR_EMPTY (str);
+	is_sensitive &= 
+		!window->last_find || 
+		(window->last_find && strcmp (window->last_find, str) != 0);
 
-	log_window_search_populate (window, str);
+	gtk_widget_set_sensitive (window->button_find, is_sensitive);
 }
 
 static void
-log_window_search_changed_cb (GtkTreeSelection *selection,
-			      GossipLogWindow  *window)
+log_window_find_changed_cb (GtkTreeSelection *selection,
+			    GossipLogWindow  *window)
 {
-	GtkTreeView      *view;
-	GtkTreeModel     *model;
-	GtkTreeIter       iter;
-
-	GossipAccount    *account;
-	GossipContact    *contact;
-	gchar            *date;
-
-	GossipContact    *own_contact;
-	GossipContact    *sender;
-	GossipMessage    *message;
-
-	GList            *messages;
-	GList            *l;
+	GtkTreeView   *view;
+	GtkTreeModel  *model;
+	GtkTreeIter    iter;
+	GossipAccount *account;
+	GossipContact *contact;
+	gchar         *date;
+	GossipContact *own_contact;
+	GossipContact *sender;
+	GossipMessage *message;
+	GList         *messages;
+	GList         *l;
+	gboolean       can_do_previous;
+	gboolean       can_do_next;
 
 	/* Get selected information */
-	view = GTK_TREE_VIEW (window->treeview_search);
+	view = GTK_TREE_VIEW (window->treeview_find);
 	model = gtk_tree_view_get_model (view);
 
 	if (!gtk_tree_selection_get_selected (selection, NULL, &iter)) {
-		gtk_widget_set_sensitive (window->button_search, FALSE);
+		gtk_widget_set_sensitive (window->button_previous, FALSE);
+		gtk_widget_set_sensitive (window->button_next, FALSE);
+
+		gossip_chat_view_clear (window->chatview_find);
+	
 		return;
 	}
 
+	gtk_widget_set_sensitive (window->button_previous, TRUE);
+	gtk_widget_set_sensitive (window->button_next, TRUE);
+
 	gtk_tree_model_get (model, &iter,
-			    COL_SEARCH_ACCOUNT, &account,
-			    COL_SEARCH_CONTACT, &contact,
-			    COL_SEARCH_DATE, &date,
+			    COL_FIND_ACCOUNT, &account,
+			    COL_FIND_CONTACT, &contact,
+			    COL_FIND_DATE, &date,
 			    -1);
 
-	gtk_widget_set_sensitive (window->button_search, TRUE);
-
 	/* Clear all current messages shown in the textview */
-	gossip_chat_view_clear (window->chatview_search);
+	gossip_chat_view_clear (window->chatview_find);
 
 	/* Turn off scrolling temporarily */
-	gossip_chat_view_scroll (window->chatview_search, FALSE);
+	gossip_chat_view_allow_scroll (window->chatview_find, FALSE);
 
 	/* Get own contact to know which messages are from me or the contact */
-	own_contact = gossip_log_get_own_contact (account);
+	own_contact = gossip_session_get_own_contact (gossip_app_get_session (),
+						      account);
 	g_object_unref (account);
 
 	/* Get messages */
-	messages = gossip_log_get_messages_for_contact (contact, date);
+	messages = gossip_log_get_messages_for_contact (window->log_manager, contact, date);
 	g_object_unref (contact);
 	g_free (date);
 
@@ -295,12 +320,12 @@ log_window_search_changed_cb (GtkTreeSelection *selection,
 		sender = gossip_message_get_sender (message);
 
 		if (gossip_contact_equal (own_contact, sender)) {
-			gossip_chat_view_append_message_from_self (window->chatview_search,
+			gossip_chat_view_append_message_from_self (window->chatview_find,
 								   message,
 								   own_contact,
 								   NULL);
 		} else {
-			gossip_chat_view_append_message_from_other (window->chatview_search,
+			gossip_chat_view_append_message_from_other (window->chatview_find,
 								    message,
 								    sender,
 								    NULL);
@@ -311,53 +336,58 @@ log_window_search_changed_cb (GtkTreeSelection *selection,
 	g_list_free (messages);
 
 	/* Scroll to the most recent messages */
-	gossip_chat_view_scroll (window->chatview_search, TRUE);
+	gossip_chat_view_allow_scroll (window->chatview_find, TRUE);
 
 	/* Highlight and find messages */
-	gossip_chat_view_highlight (window->chatview_search,
-				    window->last_search);
-	gossip_chat_view_find (window->chatview_search,
-			       window->last_search,
-			       TRUE);
+	gossip_chat_view_highlight (window->chatview_find,
+				    window->last_find);
+	gossip_chat_view_find_next (window->chatview_find,
+				    window->last_find,
+				    TRUE);
+	gossip_chat_view_find_abilities (window->chatview_find,
+					 window->last_find,
+					 &can_do_previous,
+					 &can_do_next);
+	gtk_widget_set_sensitive (window->button_previous, can_do_previous);
+	gtk_widget_set_sensitive (window->button_next, can_do_next);
+	gtk_widget_set_sensitive (window->button_find, FALSE);
 }
 
 static void
-log_window_search_populate (GossipLogWindow *window,
-			    const gchar     *search_criteria)
+log_window_find_populate (GossipLogWindow *window,
+			  const gchar     *search_criteria)
 {
-	GossipSession        *session;
-	GossipAccountManager *manager;
-	GossipAccount        *account;
-	GossipContact        *contact;
+	GossipSession      *session;
+	GossipAccount      *account;
+	GossipContact      *contact;
 
-	GList                *hits;
-	GList                *l;
-	GossipLogSearchHit   *hit;
+	GList              *hits;
+	GList              *l;
+	GossipLogSearchHit *hit;
 
-	GtkTreeView          *view;
-	GtkTreeModel         *model;
-	GtkTreeSelection     *selection;
-	GtkListStore         *store;
-	GtkTreeIter           iter;
+	GtkTreeView        *view;
+	GtkTreeModel       *model;
+	GtkTreeSelection   *selection;
+	GtkListStore       *store;
+	GtkTreeIter         iter;
 
-	view = GTK_TREE_VIEW (window->treeview_search);
+	view = GTK_TREE_VIEW (window->treeview_find);
 	model = gtk_tree_view_get_model (view);
 	selection = gtk_tree_view_get_selection (view);
 	store = GTK_LIST_STORE (model);
 
 	session = gossip_app_get_session ();
-	manager = gossip_session_get_account_manager (session);
 
-	gossip_chat_view_clear (window->chatview_search);
+	gossip_chat_view_clear (window->chatview_find);
 
 	gtk_list_store_clear (store);
 
-	if (strlen (search_criteria) == 0) {
+	if (G_STR_EMPTY (search_criteria)) {
 		/* Just clear the search. */
 		return;
 	}
 
-	hits = gossip_log_search_new (search_criteria);
+	hits = gossip_log_search_new (window->log_manager, search_criteria);
 
 	for (l = hits; l; l = l->next) {
 		const gchar *date;
@@ -378,11 +408,11 @@ log_window_search_populate (GossipLogWindow *window,
 
 		gtk_list_store_append (store, &iter);
 		gtk_list_store_set (store, &iter,
-				    COL_SEARCH_ACCOUNT, account,
-				    COL_SEARCH_CONTACT, contact,
-				    COL_SEARCH_CONTACT_NAME, gossip_contact_get_name (contact),
-				    COL_SEARCH_DATE, date,
-				    COL_SEARCH_DATE_READABLE, date_readable,
+				    COL_FIND_ACCOUNT, account,
+				    COL_FIND_CONTACT, contact,
+				    COL_FIND_CONTACT_NAME, gossip_contact_get_name (contact),
+				    COL_FIND_DATE, date,
+				    COL_FIND_DATE_READABLE, date_readable,
 				    -1);
 
 		g_free (date_readable);
@@ -394,7 +424,7 @@ log_window_search_populate (GossipLogWindow *window,
 }
 
 static void
-log_window_search_setup (GossipLogWindow *window)
+log_window_find_setup (GossipLogWindow *window)
 {
 	GtkTreeView       *view;
 	GtkTreeModel      *model;
@@ -405,11 +435,11 @@ log_window_search_setup (GossipLogWindow *window)
 	GtkCellRenderer   *cell;
 	gint               offset;
 
-	view = GTK_TREE_VIEW (window->treeview_search);
+	view = GTK_TREE_VIEW (window->treeview_find);
 	selection = gtk_tree_view_get_selection (view);
 
 	/* New store */
-	store = gtk_list_store_new (COL_SEARCH_COUNT,
+	store = gtk_list_store_new (COL_FIND_COUNT,
 				    GDK_TYPE_PIXBUF,        /* account status */
 				    GOSSIP_TYPE_ACCOUNT,    /* account */
 				    GOSSIP_TYPE_CONTACT,    /* contact */
@@ -429,7 +459,7 @@ log_window_search_setup (GossipLogWindow *window)
 	gtk_tree_view_column_pack_start (column, cell, FALSE);
 	gtk_tree_view_column_set_cell_data_func (column, cell,
 						 (GtkTreeCellDataFunc)
-						 log_window_search_pixbuf_data_func,
+						 log_window_find_pixbuf_data_func,
 						 window,
 						 NULL);
 
@@ -437,7 +467,7 @@ log_window_search_setup (GossipLogWindow *window)
 	gtk_tree_view_column_pack_start (column, cell, TRUE);
 	gtk_tree_view_column_set_cell_data_func (column, cell,
 						 (GtkTreeCellDataFunc)
-						 log_window_search_text_data_func,
+						 log_window_find_text_data_func,
 						 window,
 						 NULL);
 
@@ -449,50 +479,94 @@ log_window_search_setup (GossipLogWindow *window)
 
 	cell = gtk_cell_renderer_text_new ();
 	offset = gtk_tree_view_insert_column_with_attributes (view, -1, _("Conversation With"),
-							      cell, "text", COL_SEARCH_CONTACT_NAME,
+							      cell, "text", COL_FIND_CONTACT_NAME,
 							      NULL);
 
 	column = gtk_tree_view_get_column (view, offset - 1);
-	gtk_tree_view_column_set_sort_column_id (column, COL_SEARCH_CONTACT_NAME);
+	gtk_tree_view_column_set_sort_column_id (column, COL_FIND_CONTACT_NAME);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_clickable (column, TRUE);
 
 	cell = gtk_cell_renderer_text_new ();
 	offset = gtk_tree_view_insert_column_with_attributes (view, -1, _("Date"),
-							      cell, "text", COL_SEARCH_DATE_READABLE,
+							      cell, "text", COL_FIND_DATE_READABLE,
 							      NULL);
 
 	column = gtk_tree_view_get_column (view, offset - 1);
-	gtk_tree_view_column_set_sort_column_id (column, COL_SEARCH_DATE);
+	gtk_tree_view_column_set_sort_column_id (column, COL_FIND_DATE);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_clickable (column, TRUE);
 
 	/* Set up treeview properties */
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 	gtk_tree_sortable_set_sort_column_id (sortable,
-					      COL_SEARCH_DATE,
+					      COL_FIND_DATE,
 					      GTK_SORT_ASCENDING);
 
-	/* set up signals */
+	/* Set up signals */
 	g_signal_connect (selection, "changed",
-			  G_CALLBACK (log_window_search_changed_cb),
+			  G_CALLBACK (log_window_find_changed_cb),
 			  window);
 
 	g_object_unref (store);
 }
 
 static void
-log_window_button_search_clicked_cb (GtkWidget       *widget,
-				     GossipLogWindow *window)
+log_window_button_find_clicked_cb (GtkWidget       *widget,
+				   GossipLogWindow *window)
 {
 	const gchar *str;
 
-	str = gtk_entry_get_text (GTK_ENTRY (window->entry_search));
+	str = gtk_entry_get_text (GTK_ENTRY (window->entry_find));
 
-	if (str) {
-		gossip_chat_view_find (window->chatview_search,
-				       str,
-				       FALSE);
+	/* Don't find the same crap again */
+	if (window->last_find && strcmp (window->last_find, str) == 0) {
+		return;
+	}
+
+	g_free (window->last_find);
+	window->last_find = g_strdup (str);
+
+	log_window_find_populate (window, str);
+}
+
+static void
+log_window_button_next_clicked_cb (GtkWidget       *widget,
+				   GossipLogWindow *window)
+{
+	if (window->last_find) {
+		gboolean can_do_previous;
+		gboolean can_do_next;
+
+		gossip_chat_view_find_next (window->chatview_find,
+					    window->last_find,
+					    FALSE);
+		gossip_chat_view_find_abilities (window->chatview_find,
+						 window->last_find,
+						 &can_do_previous,
+						 &can_do_next);
+		gtk_widget_set_sensitive (window->button_previous, can_do_previous);
+		gtk_widget_set_sensitive (window->button_next, can_do_next);
+	}
+}
+
+static void
+log_window_button_previous_clicked_cb (GtkWidget       *widget,
+				       GossipLogWindow *window)
+{
+	if (window->last_find) {
+		gboolean can_do_previous;
+		gboolean can_do_next;
+
+		gossip_chat_view_find_previous (window->chatview_find,
+						window->last_find,
+						FALSE);
+		gossip_chat_view_find_abilities (window->chatview_find,
+						 window->last_find,
+						 &can_do_previous,
+						 &can_do_next);
+		gtk_widget_set_sensitive (window->button_previous, can_do_previous);
+		gtk_widget_set_sensitive (window->button_next, can_do_next);
 	}
 }
 
@@ -653,7 +727,7 @@ log_window_contacts_populate (GossipLogWindow *window)
 
 	gtk_list_store_clear (store);
 
-	contacts = gossip_log_get_contacts (account);
+	contacts = gossip_log_get_contacts (window->log_manager, account);
 
 	/* Contacts */
 	for (l = contacts; l; l = l->next) {
@@ -740,7 +814,7 @@ log_window_contacts_setup (GossipLogWindow *window)
 
 static void
 log_window_contacts_accounts_changed_cb (GtkWidget       *combobox,
-				GossipLogWindow *window)
+					 GossipLogWindow *window)
 {
 	/* Clear all current messages shown in the textview */
 	gossip_chat_view_clear (window->chatview_contacts);
@@ -771,21 +845,21 @@ log_window_contacts_new_message_cb (GossipContact   *own_contact,
 	}
 
 	/* Scroll to the most recent messages */
-	gossip_chat_view_scroll_down (window->chatview_contacts);
+	gossip_chat_view_scroll_down_smoothly (window->chatview_contacts);
 }
 
 static gboolean
 log_window_contacts_is_today_selected (GossipLogWindow *window)
 {
-	gossip_time_t  t;
-	gchar         *timestamp;
-	guint          year_selected;
-	guint          year;
-	guint          month;
-	guint          month_selected;
-	guint          day;
-	guint          day_selected;
-	gboolean       selected;
+	GossipTime  t;
+	gchar      *timestamp;
+	guint       year_selected;
+	guint       year;
+	guint       month;
+	guint       month_selected;
+	guint       day;
+	guint       day_selected;
+	gboolean    selected;
 
 	t = gossip_time_get_current ();
 	timestamp = gossip_time_to_string_local (t, "%Y%m%d");
@@ -821,16 +895,13 @@ log_window_contacts_get_messages (GossipLogWindow *window,
 	GList         *messages;
 	GList         *dates = NULL;
 	GList         *l;
-
 	const gchar   *date;
-
 	guint          year_selected;
 	guint          year;
 	guint          month;
 	guint          month_selected;
 	guint          day;
 
-
 	contact = log_window_contacts_get_selected (window);
 	if (!contact) {
 		return;
@@ -923,26 +994,29 @@ log_window_contacts_get_messages (GossipLogWindow *window,
 
 	if (log_window_contacts_is_today_selected (window)) {
 		gossip_log_handler_add_for_contact
-			(contact,
+			(window->log_manager, 
+			 contact,
 			 (GossipLogMessageFunc) log_window_contacts_new_message_cb,
 			 window);
 	} else {
 		gossip_log_handler_remove
-			((GossipLogMessageFunc) log_window_contacts_new_message_cb);
+			(window->log_manager, 
+			 (GossipLogMessageFunc) log_window_contacts_new_message_cb);
 	}
 
 	/* Clear all current messages shown in the textview */
 	gossip_chat_view_clear (window->chatview_contacts);
 
 	/* Turn off scrolling temporarily */
-	gossip_chat_view_scroll (window->chatview_search, FALSE);
+	gossip_chat_view_allow_scroll (window->chatview_contacts, FALSE);
 
 	/* Get own contact to know which messages are from me or the contact */
 	account = gossip_contact_get_account (contact);
-	own_contact = gossip_log_get_own_contact (account);
+	own_contact = gossip_session_get_own_contact (gossip_app_get_session (),
+						      account);
 
 	/* Get messages */
-	messages = gossip_log_get_messages_for_contact (contact, date);
+	messages = gossip_log_get_messages_for_contact (window->log_manager, contact, date);
 
 	for (l = messages; l; l = l->next) {
 		message = l->data;
@@ -971,7 +1045,7 @@ log_window_contacts_get_messages (GossipLogWindow *window,
 	g_object_unref (contact);
 
 	/* Turn back on scrolling */
-	gossip_chat_view_scroll (window->chatview_search, TRUE);
+	gossip_chat_view_allow_scroll (window->chatview_contacts, TRUE);
 
 	/* Scroll to the most recent messages */
 	gossip_chat_view_scroll_down (window->chatview_contacts);
@@ -1064,7 +1138,7 @@ log_window_calendar_contacts_month_changed_cb (GtkWidget       *calendar,
 
 static void
 log_window_entry_contacts_changed_cb (GtkWidget       *entry,
-				  GossipLogWindow *window)
+				      GossipLogWindow *window)
 {
 	const gchar *str;
 
@@ -1072,24 +1146,24 @@ log_window_entry_contacts_changed_cb (GtkWidget       *entry,
 	gossip_chat_view_highlight (window->chatview_contacts, str);
 
 	if (str) {
-		gossip_chat_view_find (window->chatview_contacts,
-				       str,
-				       TRUE);
+		gossip_chat_view_find_next (window->chatview_contacts,
+					    str,
+					    TRUE);
 	}
 }
 
 static void
 log_window_entry_contacts_activate_cb (GtkWidget       *entry,
-				   GossipLogWindow *window)
+				       GossipLogWindow *window)
 {
 	const gchar *str;
 
 	str = gtk_entry_get_text (GTK_ENTRY (window->entry_contacts));
 
 	if (str) {
-		gossip_chat_view_find (window->chatview_contacts,
-				       str,
-				       FALSE);
+		gossip_chat_view_find_next (window->chatview_contacts,
+					    str,
+					    FALSE);
 	}
 }
 
@@ -1098,10 +1172,10 @@ log_window_entry_contacts_activate_cb (GtkWidget       *entry,
  */
 static void
 log_window_chatrooms_pixbuf_data_func (GtkCellLayout   *cell_layout,
-				      GtkCellRenderer *cell,
-				      GtkTreeModel    *tree_model,
-				      GtkTreeIter     *iter,
-				      GossipLogWindow *window)
+				       GtkCellRenderer *cell,
+				       GtkTreeModel    *tree_model,
+				       GtkTreeIter     *iter,
+				       GossipLogWindow *window)
 {
 	GossipChatroom *chatroom;
 	GdkPixbuf      *pixbuf = NULL;
@@ -1126,10 +1200,10 @@ log_window_chatrooms_pixbuf_data_func (GtkCellLayout   *cell_layout,
 
 static void
 log_window_chatrooms_text_data_func (GtkCellLayout   *cell_layout,
-				    GtkCellRenderer *cell,
-				    GtkTreeModel    *tree_model,
-				    GtkTreeIter     *iter,
-				    GossipLogWindow *window)
+				     GtkCellRenderer *cell,
+				     GtkTreeModel    *tree_model,
+				     GtkTreeIter     *iter,
+				     GossipLogWindow *window)
 {
 	GossipChatroom *chatroom;
 
@@ -1145,7 +1219,7 @@ log_window_chatrooms_text_data_func (GtkCellLayout   *cell_layout,
 
 static void
 log_window_chatrooms_changed_cb (GtkTreeSelection *selection,
-				GossipLogWindow  *window)
+				 GossipLogWindow  *window)
 {
 	/* Use last date by default */
 	gtk_calendar_clear_marks (GTK_CALENDAR (window->calendar_chatrooms));
@@ -1243,7 +1317,7 @@ log_window_chatrooms_populate (GossipLogWindow *window)
 
 	gtk_list_store_clear (store);
 
-	chatrooms = gossip_log_get_chatrooms (account);
+	chatrooms = gossip_log_get_chatrooms (window->log_manager, account);
 
 	/* Chatrooms */
 	for (l = chatrooms; l; l = l->next) {
@@ -1360,21 +1434,21 @@ log_window_chatrooms_new_message_cb (GossipContact   *own_contact,
 	}
 
 	/* Scroll to the most recent messages */
-	gossip_chat_view_scroll_down (window->chatview_chatrooms);
+	gossip_chat_view_scroll_down_smoothly (window->chatview_chatrooms);
 }
 
 static gboolean
 log_window_chatrooms_is_today_selected (GossipLogWindow *window)
 {
-	gossip_time_t  t;
-	gchar         *timestamp;
-	guint          year_selected;
-	guint          year;
-	guint          month;
-	guint          month_selected;
-	guint          day;
-	guint          day_selected;
-	gboolean       selected;
+	GossipTime  t;
+	gchar      *timestamp;
+	guint       year_selected;
+	guint       year;
+	guint       month;
+	guint       month_selected;
+	guint       day;
+	guint       day_selected;
+	gboolean    selected;
 
 	t = gossip_time_get_current ();
 	timestamp = gossip_time_to_string_local (t, "%Y%m%d");
@@ -1405,7 +1479,6 @@ log_window_chatrooms_get_messages (GossipLogWindow *window,
 	GossipAccount  *account;
 	GossipChatroom *chatroom;
 	GossipContact  *own_contact;
-	GossipContact  *sender;
 	GossipMessage  *message;
 	GList          *messages;
 	GList          *dates = NULL;
@@ -1430,8 +1503,8 @@ log_window_chatrooms_get_messages (GossipLogWindow *window,
 	}
 
 	/* Get own contact to know which messages are from me or the contact */
-	own_contact = gossip_log_get_own_contact (account);
-
+	own_contact = gossip_session_get_own_contact (gossip_app_get_session (),
+						      account);
 	/* Either use the supplied date or get the last */
 	date = date_to_show;
 	if (!date) {
@@ -1518,39 +1591,32 @@ log_window_chatrooms_get_messages (GossipLogWindow *window,
 
 	if (log_window_chatrooms_is_today_selected (window)) {
 		gossip_log_handler_add_for_chatroom
-			(chatroom,
+			(window->log_manager, 
+			 chatroom,
 			 (GossipLogMessageFunc) log_window_chatrooms_new_message_cb,
 			 window);
 	} else {
 		gossip_log_handler_remove
-			((GossipLogMessageFunc) log_window_chatrooms_new_message_cb);
+			(window->log_manager,
+			 (GossipLogMessageFunc) log_window_chatrooms_new_message_cb);
 	}
 
 	/* Clear all current messages shown in the textview */
 	gossip_chat_view_clear (window->chatview_chatrooms);
 
 	/* Turn off scrolling temporarily */
-	gossip_chat_view_scroll (window->chatview_search, FALSE);
+	gossip_chat_view_allow_scroll (window->chatview_chatrooms, FALSE);
 
 	/* Get messages */
-	messages = gossip_log_get_messages_for_chatroom (chatroom, date);
+	messages = gossip_log_get_messages_for_chatroom (window->log_manager, chatroom, date);
 
 	for (l = messages; l; l = l->next) {
 		message = l->data;
 
-		sender = gossip_message_get_sender (message);
-
-		if (gossip_contact_equal (own_contact, sender)) {
-			gossip_chat_view_append_message_from_self (window->chatview_chatrooms,
-								   message,
-								   own_contact,
-								   NULL);
-		} else {
-			gossip_chat_view_append_message_from_other (window->chatview_chatrooms,
-								    message,
-								    sender,
-								    NULL);
-		}
+		gossip_chat_view_append_message_from_other (window->chatview_chatrooms,
+							    message,
+							    gossip_message_get_sender (message),
+							    NULL);
 	}
 
 	g_list_foreach (messages, (GFunc) g_object_unref, NULL);
@@ -1562,7 +1628,7 @@ log_window_chatrooms_get_messages (GossipLogWindow *window,
 	g_object_unref (chatroom);
 
 	/* Turn back on scrolling */
-	gossip_chat_view_scroll (window->chatview_search, TRUE);
+	gossip_chat_view_allow_scroll (window->chatview_chatrooms, TRUE);
 
 	/* Scroll to the most recent messages */
 	gossip_chat_view_scroll_down (window->chatview_chatrooms);
@@ -1661,24 +1727,24 @@ log_window_entry_chatrooms_changed_cb (GtkWidget       *entry,
 	gossip_chat_view_highlight (window->chatview_chatrooms, str);
 
 	if (str) {
-		gossip_chat_view_find (window->chatview_chatrooms,
-				       str,
-				       TRUE);
+		gossip_chat_view_find_next (window->chatview_chatrooms,
+					    str,
+					    TRUE);
 	}
 }
 
 static void
 log_window_entry_chatrooms_activate_cb (GtkWidget       *entry,
-				   GossipLogWindow *window)
+					GossipLogWindow *window)
 {
 	const gchar *str;
 
 	str = gtk_entry_get_text (GTK_ENTRY (window->entry_chatrooms));
 
 	if (str) {
-		gossip_chat_view_find (window->chatview_chatrooms,
-				       str,
-				       FALSE);
+		gossip_chat_view_find_next (window->chatview_chatrooms,
+					    str,
+					    FALSE);
 	}
 }
 
@@ -1689,12 +1755,16 @@ static void
 log_window_destroy_cb (GtkWidget       *widget,
 		       GossipLogWindow *window)
 {
-	g_free (window->last_search);
+	g_free (window->last_find);
 
 	gossip_log_handler_remove
-		((GossipLogMessageFunc) log_window_contacts_new_message_cb);
+		(window->log_manager,
+		 (GossipLogMessageFunc) log_window_contacts_new_message_cb);
 	gossip_log_handler_remove
-		((GossipLogMessageFunc) log_window_chatrooms_new_message_cb);
+		(window->log_manager,
+		 (GossipLogMessageFunc) log_window_chatrooms_new_message_cb);
+
+	g_object_unref (window->log_manager);
 
 	g_free (window);
 }
@@ -1704,11 +1774,12 @@ gossip_log_window_show (GossipContact  *contact,
 			GossipChatroom *chatroom)
 {
 	static GossipLogWindow *window = NULL;
-	GladeXML               *glade;
 	GossipSession          *session;
+	GossipLogManager       *log_manager;
+	GossipAccountChooser   *account_chooser;
 	GList                  *accounts;
 	gint                    account_num;
-	GossipAccountChooser   *account_chooser;
+	GladeXML               *glade;
 
 	if (window) {
 		gtk_window_present (GTK_WINDOW (window->window));
@@ -1726,15 +1797,20 @@ gossip_log_window_show (GossipContact  *contact,
 
 	window = g_new0 (GossipLogWindow, 1);
 
+	log_manager = gossip_session_get_log_manager (gossip_app_get_session ());
+	window->log_manager = g_object_ref (log_manager);
+
 	glade = gossip_glade_get_file ("main.glade",
 				       "log_window",
 				       NULL,
 				       "log_window", &window->window,
 				       "notebook", &window->notebook,
-				       "entry_search", &window->entry_search,
-				       "button_search", &window->button_search,
-				       "treeview_search", &window->treeview_search,
-				       "scrolledwindow_search", &window->scrolledwindow_search,
+				       "entry_find", &window->entry_find,
+				       "button_find", &window->button_find,
+				       "treeview_find", &window->treeview_find,
+				       "scrolledwindow_find", &window->scrolledwindow_find,
+				       "button_previous", &window->button_previous,
+				       "button_next", &window->button_next,
 				       "entry_contacts", &window->entry_contacts,
 				       "calendar_contacts", &window->calendar_contacts,
 				       "vbox_contacts", &window->vbox_contacts,
@@ -1746,12 +1822,13 @@ gossip_log_window_show (GossipContact  *contact,
 				       "treeview_chatrooms", &window->treeview_chatrooms,
 				       "scrolledwindow_chatrooms", &window->scrolledwindow_chatrooms,
 				       NULL);
-
 	gossip_glade_connect (glade,
 			      window,
 			      "log_window", "destroy", log_window_destroy_cb,
-			      "entry_search", "activate", log_window_entry_search_activate_cb,
-			      "button_search", "clicked", log_window_button_search_clicked_cb,
+			      "entry_find", "changed", log_window_entry_find_changed_cb,
+			      "button_previous", "clicked", log_window_button_previous_clicked_cb,
+			      "button_next", "clicked", log_window_button_next_clicked_cb,
+			      "button_find", "clicked", log_window_button_find_clicked_cb,
 			      "entry_contacts", "changed", log_window_entry_contacts_changed_cb,
 			      "entry_contacts", "activate", log_window_entry_contacts_activate_cb,
 			      "entry_chatrooms", "changed", log_window_entry_chatrooms_changed_cb,
@@ -1779,10 +1856,10 @@ gossip_log_window_show (GossipContact  *contact,
 			  window);
 
 	/* Configure Search GossipChatView */
-	window->chatview_search = gossip_chat_view_new ();
-	gtk_container_add (GTK_CONTAINER (window->scrolledwindow_search),
-			   GTK_WIDGET (window->chatview_search));
-	gtk_widget_show (GTK_WIDGET (window->chatview_search));
+	window->chatview_find = gossip_chat_view_new ();
+	gtk_container_add (GTK_CONTAINER (window->scrolledwindow_find),
+			   GTK_WIDGET (window->chatview_find));
+	gtk_widget_show (GTK_WIDGET (window->chatview_find));
 
 	/* Configure Contacts GossipChatView */
 	window->chatview_contacts = gossip_chat_view_new ();
@@ -1842,7 +1919,7 @@ gossip_log_window_show (GossipContact  *contact,
 	}
 
 	/* Search List */
-	log_window_search_setup (window);
+	log_window_find_setup (window);
 
 	/* Contacts */
 	log_window_contacts_setup (window);
