@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2005-2006 Imendio AB
+ * Copyright (C) 2005-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,21 +18,26 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
 #include <stdlib.h>
+
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <glade/glade.h>
+
+#include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-stock.h>
 
 #include "gossip-marshal.h"
 #include "gossip-app.h"
+#include "gossip-glade.h"
 #include "gossip-ui-utils.h"
-#include "gossip-stock.h"
 #include "gossip-presence-chooser.h"
 #include "gossip-status-presets.h"
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
-		       GOSSIP_TYPE_PRESENCE_CHOOSER, GossipPresenceChooserPriv))
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_PRESENCE_CHOOSER, GossipPresenceChooserPriv))
 
 typedef struct {
 	GtkWidget           *hbox;
@@ -43,17 +48,17 @@ typedef struct {
 
 	GossipPresenceState  last_state;
 
-	guint                flash_interval;
+	guint                heartbeat_id;
 
 	GossipPresenceState  flash_state_1;
 	GossipPresenceState  flash_state_2;
 
-	guint                flash_timeout_id;
-
 	/* The handle the kind of unnessecary scroll support. */
 	guint                scroll_timeout_id;
 	GossipPresenceState  scroll_state;
 	gchar               *scroll_status;
+
+	GtkTooltips         *tooltips;
 } GossipPresenceChooserPriv;
 
 static void     presence_chooser_finalize               (GObject               *object);
@@ -98,7 +103,15 @@ static gboolean presence_chooser_button_press_event_cb  (GtkWidget             *
 static gboolean presence_chooser_scroll_event_cb        (GtkWidget             *chooser,
 							 GdkEventScroll        *event,
 							 gpointer               user_data);
-static gboolean presence_chooser_flash_timeout_cb       (GossipPresenceChooser *chooser);
+static void     presence_chooser_size_allocate_cb       (GtkWidget             *chooser,
+							 GtkAllocation         *allocation,
+							 gpointer               user_data);
+static gboolean presence_chooser_flash_heartbeat_func   (GossipHeartbeat       *heartbeat,
+							 gpointer               user_data);
+static void     presence_chooser_flash_start_cb         (GossipSelfPresence    *self_presence,
+							 GossipPresenceChooser *chooser);
+static void     presence_chooser_flash_stop_cb          (GossipSelfPresence    *self_presence,
+							 GossipPresenceChooser *chooser);
 
 G_DEFINE_TYPE (GossipPresenceChooser, gossip_presence_chooser, GTK_TYPE_TOGGLE_BUTTON);
 
@@ -138,9 +151,6 @@ gossip_presence_chooser_init (GossipPresenceChooser *chooser)
 
 	priv = GET_PRIV (chooser);
 
-	/* Default to 1/2 a second flash interval */
-	priv->flash_interval = 500;
-
 	gtk_button_set_relief (GTK_BUTTON (chooser), GTK_RELIEF_NONE);
 	gtk_button_set_focus_on_click (GTK_BUTTON (chooser), FALSE);
 
@@ -172,6 +182,8 @@ gossip_presence_chooser_init (GossipPresenceChooser *chooser)
 	gtk_widget_show (arrow);
 	gtk_container_add (GTK_CONTAINER (alignment), arrow);
 
+	priv->tooltips = gtk_tooltips_new ();
+
 	g_signal_connect (chooser, "toggled",
 			  G_CALLBACK (presence_chooser_toggled_cb),
 			  NULL);
@@ -181,6 +193,17 @@ gossip_presence_chooser_init (GossipPresenceChooser *chooser)
 	g_signal_connect (chooser, "scroll-event",
 			  G_CALLBACK (presence_chooser_scroll_event_cb),
 			  NULL);
+	g_signal_connect (chooser, "size-allocate",
+			  G_CALLBACK (presence_chooser_size_allocate_cb),
+			  NULL);
+
+	g_signal_connect (gossip_app_get_self_presence (), "start-flash",
+			  G_CALLBACK (presence_chooser_flash_start_cb),
+			  chooser);
+
+	g_signal_connect (gossip_app_get_self_presence (), "stop-flash",
+			  G_CALLBACK (presence_chooser_flash_stop_cb),
+			  chooser);
 }
 
 static void
@@ -190,14 +213,23 @@ presence_chooser_finalize (GObject *object)
 
 	priv = GET_PRIV (object);
 
-	if (priv->flash_timeout_id) {
-		g_source_remove (priv->flash_timeout_id);
+	if (priv->heartbeat_id) {
+		gossip_heartbeat_callback_remove (gossip_app_get_flash_heartbeat (), 
+						  priv->heartbeat_id);
 	}
 
 	if (priv->scroll_timeout_id) {
 		g_source_remove (priv->scroll_timeout_id);
 	}
 
+	g_signal_handlers_disconnect_by_func (gossip_app_get_self_presence (),
+					      presence_chooser_flash_stop_cb,
+					      object);
+
+	g_signal_handlers_disconnect_by_func (gossip_app_get_self_presence (),
+					      presence_chooser_flash_start_cb,
+					      object);
+
 	G_OBJECT_CLASS (gossip_presence_chooser_parent_class)->finalize (object);
 }
 
@@ -230,7 +262,7 @@ presence_chooser_set_state (GossipPresenceChooser *chooser,
 
 	default_status = gossip_presence_state_get_default_status (state);
 
-	if (strlen (status) < 1) {
+	if (G_STR_EMPTY (status)) {
 		status = default_status;
 	} else {
 		/* Only store the value if it differs from the default ones. */
@@ -241,6 +273,10 @@ presence_chooser_set_state (GossipPresenceChooser *chooser,
 
 	priv->last_state = state;
 
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (priv->tooltips), 
+			      GTK_WIDGET (chooser), 
+			      status, status);
+
 	presence_chooser_reset_scroll_timeout (chooser);
 	g_signal_emit (chooser, signals[CHANGED], 0, state, status);
 }
@@ -408,12 +444,19 @@ static void
 presence_chooser_noncustom_activate_cb (GtkWidget             *item,
 					GossipPresenceChooser *chooser)
 {
-	GossipPresenceState  state;
-	const gchar         *status;
+	GossipPresenceChooserPriv *priv;
+	GossipPresenceState        state;
+	const gchar               *status;
+
+	priv = GET_PRIV (chooser);
 
 	status = g_object_get_data (G_OBJECT (item), "status");
 	state = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "state"));
 
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (priv->tooltips), 
+			      GTK_WIDGET (chooser), 
+			      status, status);
+
 	presence_chooser_reset_scroll_timeout (chooser);
 	g_signal_emit (chooser, signals[CHANGED], 0, state, status);
 }
@@ -771,6 +814,72 @@ presence_chooser_scroll_timeout_cb (GossipPresenceChooser *chooser)
 	return FALSE;
 }
 
+static void
+presence_chooser_flash_start (GossipPresenceChooser *chooser,
+			      GossipPresenceState    state_1,
+			      GossipPresenceState    state_2)
+{
+	GossipPresenceChooserPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_PRESENCE_CHOOSER (chooser));
+
+	priv = GET_PRIV (chooser);
+
+	if (priv->heartbeat_id) {
+		/* Already blinking */
+		return;
+	}
+
+	priv->flash_state_1 = state_1;
+	priv->flash_state_2 = state_2;
+
+	priv->heartbeat_id = gossip_heartbeat_callback_add (gossip_app_get_flash_heartbeat (),
+							    presence_chooser_flash_heartbeat_func,
+							    chooser);
+}
+
+static void
+presence_chooser_flash_stop (GossipPresenceChooser *chooser,
+			     GossipPresenceState    state)
+{
+	GossipPresenceChooserPriv *priv;
+	GdkPixbuf                 *pixbuf;
+
+	g_return_if_fail (GOSSIP_IS_PRESENCE_CHOOSER (chooser));
+
+	priv = GET_PRIV (chooser);
+
+	if (priv->heartbeat_id) {
+		gossip_heartbeat_callback_remove (gossip_app_get_flash_heartbeat (),
+						  priv->heartbeat_id);
+		priv->heartbeat_id = 0;
+	}
+
+	pixbuf = gossip_pixbuf_for_presence_state (state);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (priv->image), pixbuf);
+	g_object_unref (pixbuf);
+
+	priv->last_state = state;
+}
+
+static void
+presence_chooser_flash_start_cb (GossipSelfPresence    *self_presence,
+				 GossipPresenceChooser *chooser)
+{
+	presence_chooser_flash_start (chooser,
+				      gossip_self_presence_get_current_state (gossip_app_get_self_presence ()),
+				      gossip_self_presence_get_previous_state (gossip_app_get_self_presence ()));
+	
+}
+
+static void
+presence_chooser_flash_stop_cb (GossipSelfPresence    *self_presence,
+				GossipPresenceChooser *chooser)
+{
+	presence_chooser_flash_stop (chooser,
+				     gossip_self_presence_get_current_state (gossip_app_get_self_presence ()));
+}
+
 static gboolean
 presence_chooser_scroll_event_cb (GtkWidget      *chooser,
 				  GdkEventScroll *event,
@@ -856,6 +965,25 @@ presence_chooser_scroll_event_cb (GtkWidget      *chooser,
 	return TRUE;
 }
 
+static void
+presence_chooser_size_allocate_cb (GtkWidget     *chooser,
+				   GtkAllocation *allocation,
+				   gpointer       user_data)
+{
+	GossipPresenceChooserPriv *priv;
+	PangoLayout               *layout;
+
+	priv = GET_PRIV (chooser);
+
+	layout = gtk_label_get_layout (GTK_LABEL (priv->label));
+
+	if (pango_layout_is_ellipsized (layout)) {
+		gtk_tooltips_enable (priv->tooltips);
+	} else {
+		gtk_tooltips_disable (priv->tooltips);
+	}
+}
+
 GtkWidget *
 gossip_presence_chooser_new (void)
 {
@@ -981,7 +1109,7 @@ gossip_presence_chooser_set_state (GossipPresenceChooser *chooser,
 
 	priv = GET_PRIV (chooser);
 
-	gossip_presence_chooser_flash_stop (chooser, state);
+	presence_chooser_flash_stop (chooser, state);
 }
 
 void
@@ -995,24 +1123,14 @@ gossip_presence_chooser_set_status (GossipPresenceChooser *chooser,
 	priv = GET_PRIV (chooser);
 
 	gtk_label_set_text (GTK_LABEL (priv->label), status);
+
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (priv->tooltips), 
+			      GTK_WIDGET (chooser), 
+			      status, status);
 }
 
-void
-gossip_presence_chooser_set_flash_interval (GossipPresenceChooser *chooser,
-					    guint                  ms)
-{
-	GossipPresenceChooserPriv *priv;
-
-	g_return_if_fail (GOSSIP_IS_PRESENCE_CHOOSER (chooser));
-	g_return_if_fail (ms > 1 && ms < 30000);
-
-	priv = GET_PRIV (chooser);
-
-	priv->flash_interval = ms;
-}
-
-static gboolean
-presence_chooser_flash_timeout_cb (GossipPresenceChooser *chooser)
+static void
+presence_chooser_flash (GossipPresenceChooser *chooser)
 {
 	GossipPresenceChooserPriv *priv;
 	GossipPresenceState        state;
@@ -1021,7 +1139,7 @@ presence_chooser_flash_timeout_cb (GossipPresenceChooser *chooser)
 
 	priv = GET_PRIV (chooser);
 
-	if (on) {
+	if (!on) {
 		state = priv->flash_state_1;
 	} else {
 		state = priv->flash_state_2;
@@ -1032,68 +1150,14 @@ presence_chooser_flash_timeout_cb (GossipPresenceChooser *chooser)
 	g_object_unref (pixbuf);
 
 	on = !on;
+}
 
+static gboolean
+presence_chooser_flash_heartbeat_func (GossipHeartbeat *heartbeat,
+				       gpointer         user_data)
+{
+	presence_chooser_flash (GOSSIP_PRESENCE_CHOOSER (user_data));
+	
 	return TRUE;
 }
 
-void
-gossip_presence_chooser_flash_start (GossipPresenceChooser *chooser,
-				     GossipPresenceState    state_1,
-				     GossipPresenceState    state_2)
-{
-	GossipPresenceChooserPriv *priv;
-
-	g_return_if_fail (GOSSIP_IS_PRESENCE_CHOOSER (chooser));
-
-	priv = GET_PRIV (chooser);
-
-	if (priv->flash_timeout_id != 0) {
-		return;
-	}
-
-	priv->flash_state_1 = state_1;
-	priv->flash_state_2 = state_2;
-
-	priv->flash_timeout_id = g_timeout_add (priv->flash_interval,
-						(GSourceFunc) presence_chooser_flash_timeout_cb,
-						chooser);
-}
-
-void
-gossip_presence_chooser_flash_stop (GossipPresenceChooser *chooser,
-				    GossipPresenceState    state)
-{
-	GossipPresenceChooserPriv *priv;
-	GdkPixbuf                 *pixbuf;
-
-	g_return_if_fail (GOSSIP_IS_PRESENCE_CHOOSER (chooser));
-
-	priv = GET_PRIV (chooser);
-
-	if (priv->flash_timeout_id) {
-		g_source_remove (priv->flash_timeout_id);
-		priv->flash_timeout_id = 0;
-	}
-
-	pixbuf = gossip_pixbuf_for_presence_state (state);
-	gtk_image_set_from_pixbuf (GTK_IMAGE (priv->image), pixbuf);
-	g_object_unref (pixbuf);
-
-	priv->last_state = state;
-}
-
-gboolean
-gossip_presence_chooser_is_flashing (GossipPresenceChooser *chooser)
-{
-	GossipPresenceChooserPriv *priv;
-
-	g_return_val_if_fail (GOSSIP_IS_PRESENCE_CHOOSER (chooser), FALSE);
-
-	priv = GET_PRIV (chooser);
-
-	if (priv->flash_timeout_id) {
-		return TRUE;
-	}
-
-	return FALSE;
-}
